/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import type { Linter } from "eslint";

const maxComplexity = 10;
const maxClasses = 1;
const maxDepth = 5;
const maxCallbacks = 2;
const maxStatements = 20;

/**
 * Rules explicitly disabled to be overridden
 * by external packages.
 */
const disabledEslintRules: Linter.RulesRecord = {
  "class-methods-use-this":       "off",
  "default-param-last":           "off",
  "dot-notation":                 "off",
  "init-declarations":            "off",
  "max-params":                   "off",
  "no-array-constructor":         "off",
  "no-empty-function":            "off",
  "no-implied-eval":              "off",
  "no-loop-func":                 "off",
  "no-loss-of-precision":         "off",
  "no-return-await":              "off",
  "no-shadow":                    "off",
  "no-throw-literal":             "off",
  "no-unused-expressions":        "off",
  "no-unused-vars":               "off",
  "no-use-before-define":         "off",
  "no-useless-constructor":       "off",
  "prefer-destructuring":         "off",
  "prefer-promise-reject-errors": "off",
  "require-await":                "off",
};

/**
 * Rules for the base eslint.
 */
const eslintRules: Linter.RulesRecord = {
  "accessor-pairs":        "error",
  "array-callback-return": [ "error", { allowImplicit: true } ],
  "arrow-body-style":      [ "warn", "always" ],
  "block-scoped-var":      "error",
  "capitalized-comments":  "warn",
  "complexity":            [ "error", maxComplexity ],
  "consistent-return":     "error",
  "curly":                 "error",
  "default-case":          "error",
  "default-case-last":     "warn",
  "eqeqeq":                "error",
  "for-direction":         "error",
  "func-name-matching":    "error",
  "func-names":            [ "warn", "always" ],
  "func-style":
  [ "warn", "declaration", { allowArrowFunctions: true } ],
  "getter-return":                [ "error", { allowImplicit: false } ],
  "grouped-accessor-pairs":       "warn",
  "logical-assignment-operators": [ "error", "never" ],
  "max-classes-per-file":         [ "error", maxClasses ],
  "max-depth":                    [ "warn", maxDepth ],
  "max-lines":
  [ "warn", { max: 300, skipBlankLines: true, skipComments: true } ],
  "max-lines-per-function": [
    "warn",
    { IIFEs: true, max: 50, skipBlankLines: true, skipComments: false },
  ],
  "max-nested-callbacks":            [ "warn", maxCallbacks ],
  "max-statements":                  [ "warn", maxStatements ],
  "new-cap":                         "warn",
  "no-alert":                        "warn",
  "no-async-promise-executor":       "error",
  "no-await-in-loop":                "warn",
  "no-bitwise":                      "error",
  "no-caller":                       "error",
  "no-case-declarations":            "warn",
  "no-class-assign":                 "error",
  "no-compare-neg-zero":             "error",
  "no-cond-assign":                  "error",
  "no-console":                      "warn",
  // Technically covered by ts
  "no-const-assign":                 "error",
  "no-constant-binary-expression":   "error",
  "no-constant-condition":           "error",
  "no-constructor-return":           "error",
  "no-control-regex":                "error",
  "no-debugger":                     "warn",
  "no-delete-var":                   "error",
  "no-div-regex":                    "warn",
  "no-dupe-args":                    "error",
  "no-dupe-class-members":           "error",
  "no-dupe-else-if":                 "error",
  "no-dupe-keys":                    "error",
  "no-duplicate-case":               "error",
  "no-duplicate-imports":            "warn",
  "no-else-return":                  "warn",
  "no-empty":                        "warn",
  "no-empty-character-class":        "warn",
  "no-empty-pattern":                "warn",
  "no-empty-static-block":           "warn",
  "no-eq-null":                      "error",
  "no-eval":                         "error",
  "no-ex-assign":                    "error",
  "no-extend-native":                "error",
  "no-extra-bind":                   "warn",
  "no-extra-boolean-cast":           "warn",
  "no-extra-label":                  "warn",
  "no-fallthrough":                  "error",
  "no-func-assign":                  "error",
  "no-global-assign":                "error",
  "no-implicit-coercion":            "warn",
  "no-implicit-globals":             "error",
  "no-import-assign":                "error",
  "no-inner-declarations":           "warn",
  "no-invalid-regexp":               "warn",
  "no-invalid-this":                 "error",
  "no-irregular-whitespace":         "error",
  "no-iterator":                     "error",
  "no-label-var":                    "error",
  "no-labels":                       "warn",
  "no-lone-blocks":                  "warn",
  "no-lonely-if":                    "warn",
  "no-misleading-character-class":   "warn",
  "no-multi-assign":                 "warn",
  "no-multi-str":                    "warn",
  "no-negated-condition":            "warn",
  "no-nested-ternary":               "warn",
  "no-new":                          "warn",
  "no-new-native-nonconstructor":    "error",
  "no-new-wrappers":                 "error",
  "no-nonoctal-decimal-escape":      "warn",
  "no-obj-calls":                    "warn",
  "no-object-constructor":           "error",
  "no-octal":                        "warn",
  "no-octal-escape":                 "warn",
  "no-param-reassign":               "error",
  "no-plusplus":                     "warn",
  "no-promise-executor-return":      "warn",
  "no-proto":                        "warn",
  "no-prototype-builtins":           "error",
  "no-regex-spaces":                 "warn",
  "no-return-assign":                "error",
  "no-script-url":                   "warn",
  "no-self-assign":                  "error",
  "no-self-compare":                 "error",
  "no-sequences":                    "warn",
  "no-setter-return":                "error",
  "no-sparse-arrays":                "error",
  "no-template-curly-in-string":     "warn",
  "no-this-before-super":            "error",
  "no-undef":                        "error",
  "no-undef-init":                   "warn",
  "no-underscore-dangle":            "warn",
  "no-unexpected-multiline":         "warn",
  "no-unmodified-loop-condition":    "warn",
  "no-unneeded-ternary":             "warn",
  "no-unreachable":                  "warn",
  "no-unreachable-loop":             "warn",
  "no-unsafe-finally":               "error",
  "no-unsafe-negation":              "error",
  "no-unsafe-optional-chaining":     "error",
  "no-unused-private-class-members": "warn",
  "no-useless-assignment":           "warn",
  "no-useless-backreference":        "warn",
  "no-useless-call":                 "warn",
  "no-useless-catch":                "warn",
  "no-useless-computed-key":         "warn",
  "no-useless-concat":               "warn",
  "no-useless-escape":               "warn",
  "no-useless-rename":               "warn",
  "no-useless-return":               "warn",
  "no-var":                          "error",
  "no-warning-comments":             "warn",
  "no-with":                         "error",
  "object-shorthand":                [ "warn", "consistent-as-needed" ],
  "operator-assignment":             [ "warn", "never" ],
  "prefer-arrow-callback":           "warn",
  "prefer-const":                    "error",
  "prefer-named-capture-group":      "warn",
  "prefer-object-has-own":           "warn",
  "prefer-object-spread":            "warn",
  "prefer-regex-literals":           "warn",
  "prefer-rest-params":              "warn",
  "prefer-spread":                   "warn",
  "prefer-template":                 "warn",
  "radix":                           "error",
  "require-atomic-updates":          "error",
  "require-yield":                   "warn",
  "sort-keys":                       "warn",
  "sort-vars":                       "warn",
  "symbol-description":              "warn",
  "use-isnan":                       "warn",
  "valid-typeof":                    "error",
  "yoda":                            [ "warn", "never" ],
};

export { disabledEslintRules, eslintRules };
